


def get_deprecated_kos():
    deprecated_kos = [
        'K11189',  # should be K02784
    ]
    return deprecated_kos



def get_rids_with_mancheck_gpr():
    rids_mancheck_gpr = [  # reactions with manually checked GPRs
        'SUCD1', 'ALKP', 'PFK_3', 'TCMPTS', 'PPA', 'APSR',
        'NADHPO', 'ACOD', '5DOAN', '5DRBK', '5DRBPI', '5DRBUPD',
        'FNPOR', 'THZPSN', 'PPTGP', 'ALATA_D2', 'XYLUR1'
    ]
    return rids_mancheck_gpr



def get_manual_sinks():
    
    return ['apoACP', 'aponit', 'apocarb', 'thioca', 'THI5p_b', 'cyE']



def get_manual_demands():
    
    return ['scp', 'amob', 'dialurate', 'THI5p_a', 'partmass' ]



def get_custom_groups():
    
    
    return {
        'gr_ptdSTA': ['UAMAGLL', 'UMNS', 'UPPNAPT', 'UAGPT2', 'UAGPGAL', 'UAGPN6GT', 'UAAGGTLGAGT', 'UAAGGTLG3AGT', 'PPTGP'],
        'gr_HemeO': ['HEMEOS'],
        'gr_WTA1': ['ACGAMT', 'UNDBD', 'WTAGPT', 'WTAGPP', 'WTAUGLCT2', 'WTAALAT3', 'WTAPL3'],
        'gr_WTA2': ['ACGAMT', 'UNDBD', 'WTAGPT', 'WTARBT2', 'WTARPP2', 'WTAUGLCT', 'WTAALAT2', 'WTAPL2'],
        'gr_WTA3': ['ACGAMT', 'UNDBD', 'WTAGPT', 'WTAGPT2', 'WTARPP', 'WTAGLCNACT', 'WTAALAT', 'WTAPL'],
        'gr_LTA1': ['UGDIAT', 'UGLDIAT', 'GGGDAGF2', 'LIPOPO2', 'LTANACT', 'LTAALAT2'],
        'gr_LTA2': ['UGDIAT', 'UGADIAT', 'GGGDAGF', 'LIPOPO', 'LTAGAT', 'LTAALAT'],
        'gr_br': ['LYEH1', 'HIPCD1', 'LYEH2', 'HIPCD2', 'BHBRH1', 'BHBRH2'],
        'gr_PHA1': ['ACACT1r', 'AACOAR_syn', 'PHBS_syn_1', 'PHBDEP_1'],    # PHA from glycolyis
    }